package com.example.ttdvaadin;

import java.util.ArrayList;
import java.util.List;

import com.example.ttdvaadin.domain.Task;
import com.example.ttdvaadin.util.DateBeforeValidator;
import com.vaadin.data.Item;
import com.vaadin.data.util.BeanItem;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Component;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.FormFieldFactory;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.NativeSelect;
import com.vaadin.ui.PopupDateField;
import com.vaadin.ui.TextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;

public class TaskFormWindowFactory extends Form implements FormFieldFactory {

	private static final long serialVersionUID = 1L;

	List<String> visibleFields = new ArrayList<String>();

	Button actionButton;
	Button closeButton = new Button("Close");

	Window modalWindow;

	BeanItem<Task> taskBeanItem;

	VaadinApp vaadinApp;

	public TaskFormWindowFactory(BeanItem<Task> taskBeanItem,
			VaadinApp vaadinApp, String action) {

		this.vaadinApp = vaadinApp;

		GridLayout gl = new GridLayout(3, 4);
		gl.setSpacing(true);

		setLayout(gl);

		this.taskBeanItem = taskBeanItem;
		if(action.equals("new")) this.actionButton = new Button("Add task");
		else this.actionButton = new Button("Edit task");
		
		visibleFields.add("id");
		visibleFields.add("title");
		visibleFields.add("taskBody");
		visibleFields.add("deadline");
		visibleFields.add("importance");

		setImmediate(true);
		setValidationVisibleOnCommit(true);

		// 1.
		setFormFieldFactory(this);
		// 2.
		setItemDataSource(taskBeanItem);
		// 3.
		setVisibleItemProperties(visibleFields);

		closeButton.addListener(ClickEvent.class, this, "closeWindowAction");
		
		if(action.equals("new")) actionButton.addListener(ClickEvent.class, this, "addTaskAction");
		else actionButton.addListener(ClickEvent.class, this, "editTaskAction");
		gl.addComponent(actionButton, 0, 3);
		gl.addComponent(closeButton, 1, 3);
	}

	@Override
	protected void attachField(Object propertyId, Field field) {
		String property = (String) propertyId;

		GridLayout gl = (GridLayout) getLayout();

		if ("title".equals(property)) {
			gl.addComponent(field,0,0,0,0);
		} else if ("taskBody".equals(property)) {
			gl.addComponent(field,0,2,2,2);
		} else if ("deadline".equals(property)) {
			gl.addComponent(field,1,0,1,0);
		} else if ("importance".equals(property)) {
			gl.addComponent(field,0,1,1,1);
		}
	}

	@Override
	public Field createField(Item item, Object propertyId, Component uiContext) {

		String field = (String) propertyId;

		if ("title".equals(field)) {
			TextField titleTextField = new TextField("Task Title");
			titleTextField.setRequired(true);
			titleTextField.setInputPrompt("Please type task title");
			titleTextField.setWidth("200px");
			return titleTextField;

		} else if ("taskBody".equals(field)) {

			TextArea taskBodyTextArea = new TextArea("Task Body");
			taskBodyTextArea.setColumns(30);
			taskBodyTextArea.setRows(10);
			taskBodyTextArea.setRequired(true);
			taskBodyTextArea.setInputPrompt("Please type task body");
			return taskBodyTextArea;

		} else if ("deadline".equals(field)) {

			PopupDateField deadlineDate = new PopupDateField("Deadline");
			deadlineDate.setInputPrompt("Please type deadline date");
			deadlineDate.setWidth("200px");
			deadlineDate.setResolution(PopupDateField.RESOLUTION_DAY);
			deadlineDate.setParseErrorMessage("Date format is not recognized. Please eneter the date in format: dd.MM.yyyy");
			deadlineDate.setRequired(true);
			deadlineDate.addValidator(new DateBeforeValidator());
			deadlineDate.setImmediate(true);

			return deadlineDate;

		} else if ("importance".equals(field)) {

			final String[] selectOptions = new String[] { "not important",
					"a little important", "important", "very important",
					"very very much important" };

			NativeSelect importanceNativeSelect = new NativeSelect(
					"Please select an importance");
			for (String i : selectOptions) {
				importanceNativeSelect.addItem(i);
			}

			importanceNativeSelect.setNullSelectionAllowed(false);
			importanceNativeSelect.setValue("not important");
			importanceNativeSelect.setImmediate(true);
			return importanceNativeSelect;
		}
		return null;
	}

	public Window createWindow() {
		modalWindow = new Window();
		modalWindow.setModal(true);
		modalWindow.setClosable(false);
		modalWindow.addComponent(this);

		((VerticalLayout) modalWindow.getContent()).setSizeUndefined();
		((VerticalLayout) modalWindow.getContent()).setMargin(true);
		((VerticalLayout) modalWindow.getContent()).setSpacing(true);

		return modalWindow;
	}

	public void closeWindowAction(ClickEvent event) {
		closeFormWindow();
		vaadinApp.updateTtdTable();
	}

	public void addTaskAction(ClickEvent event) {
		commit();
		Window mainWindow = modalWindow.getParent();
		Task task = taskBeanItem.getBean();
		vaadinApp.getStorageService().addTask(task);
		closeFormWindow();
		vaadinApp.updateTtdTable();
		mainWindow.showNotification("Task was added successfully");
	}
	
	public void editTaskAction(ClickEvent event) {
		commit();
		Window mainWindow = modalWindow.getParent();
		
		Task task = taskBeanItem.getBean();
		vaadinApp.getStorageService().editTask(task);
		closeFormWindow();
		vaadinApp.updateTtdTable();
		mainWindow.showNotification("Task was updated successfully");
	}

	private void closeFormWindow() {
		Window mainWindow = modalWindow.getParent();
		mainWindow.removeWindow(modalWindow);
	}

}
