package com.example.ttdvaadin;
import org.vaadin.dialogs.ConfirmDialog;

import com.example.ttdvaadin.domain.Task;
import com.example.ttdvaadin.service.StorageService;
import com.vaadin.Application;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.Table;
import com.vaadin.ui.Window;

public class VaadinApp extends Application {

	private static final long serialVersionUID = 1L;

	private BeanItemContainer<Task> taskContainer = new BeanItemContainer<Task>(
			Task.class);
	private Table ttdTable = new Table("Things To Do", taskContainer);

	private Window mainWindow = new Window();
	private GridLayout gl = new GridLayout(3,2);
	private Button addTaskButton = new Button("Add new Task");
	private Button deleteTaskButton = new Button("Delete selected Task");
	private Button editTaskButton = new Button("Edit selected Task");
	private Window taskFormWindow;
	private StorageService storageService;

	public VaadinApp(StorageService storageService) {
		this.storageService = storageService;
	}

	public StorageService getStorageService() {
		return storageService;
	}

	@Override
	public void init() {
		initDataSource();
		updateTtdTable();


		ttdTable.setSelectable(true);
		ttdTable.setImmediate(true);
		ttdTable.setVisible(true);
		ttdTable.setVisibleColumns(new String[]{"title","taskBody","importance","deadline"});
		
		addTaskButton.addListener(new Button.ClickListener() {

			private static final long serialVersionUID = 1L;

			@Override
			public void buttonClick(ClickEvent event) {

				if (taskFormWindow != null) {
					mainWindow.removeWindow(taskFormWindow);
				}

				BeanItem<Task> beanTaskItem = new BeanItem<Task>(new Task());
				TaskFormWindowFactory tff = new TaskFormWindowFactory(
						beanTaskItem, VaadinApp.this,"new");

				taskFormWindow = tff.createWindow();
				mainWindow.addWindow(taskFormWindow);
			}
		});

		editTaskButton.addListener(new Button.ClickListener() {

			private static final long serialVersionUID = 1L;

			@Override
			public void buttonClick(ClickEvent event) {
				if (ttdTable.getValue() != null) {

					if (taskFormWindow != null) {
						mainWindow.removeWindow(taskFormWindow);
					}

					BeanItem<Task> beanTaskItem = new BeanItem<Task>((Task)ttdTable.getValue());
					TaskFormWindowFactory tff = new TaskFormWindowFactory(
							beanTaskItem, VaadinApp.this,"edit");

					taskFormWindow = tff.createWindow();
					mainWindow.addWindow(taskFormWindow);
					
					updateTtdTable();
				} else
					mainWindow
							.showNotification("Please select task to update !");
			}
		});

		deleteTaskButton.addListener(new Button.ClickListener() {

			private static final long serialVersionUID = 1L;

			@Override
			public void buttonClick(ClickEvent event) {
				
				final Task selected = (Task) ttdTable.getValue();
				
				if (selected != null) {
					
					ConfirmDialog.show(getMainWindow(),"Please Confirm:", "Do you really want to delete Task titled: "+selected.getTitle(),
					        "Yes I want", "Not quite", new ConfirmDialog.Listener() {
								private static final long serialVersionUID = 1L;

								public void onClose(ConfirmDialog dialog) {
					                if (dialog.isConfirmed()) {
					                    // Confirmed to continue
					                	storageService.deleteTask(selected);
										updateTtdTable();
					                } else {
					                    // User did not confirm
					                   
					                }
					            }
					        });
					
				} else
					mainWindow
							.showNotification("Please select task to remove !");
			}
		});
		gl.addComponent(ttdTable,0,0,2,0);
		gl.addComponent(addTaskButton,0,1,0,1);
		gl.addComponent(editTaskButton,1,1,1,1);
		gl.addComponent(deleteTaskButton,2,1,2,1);
		mainWindow.addComponent(gl);
		setMainWindow(mainWindow);
	}
	
	public void updateTtdTable() {
		taskContainer.removeAllItems();
		taskContainer.addAll(storageService.getAllTasks());
	}

	private void initDataSource() {

		Task t1 = new Task("Projekt Vaadin",
				"Do napisania aplikacja w Vaadinie na piątek (30.11.2012).",
				"30.11.2012", 4);
		Task t2 = new Task("Sprzęt w NEO24 do odbioru",
				"Do odbioru sprzęt. Odebrać w poniedziałek w Punkcie wydań",
				"26.11.2012", 2);
		Task t3 = new Task("Montaż alarmu",
				"Umówiony montaż alarmu na wtorek godz. 10:00", "27.11.2012", 5);
		Task t4 = new Task(
				"Montaż mebli kuchennych",
				"W środę 28.11.2012 planowany jest przyjazd ekipy z meblami. PotWierdzić !!",
				"28.11.2012", 5);
		Task t5 = new Task(
				"Mnozenie macierzy na CPU i GPU",
				"Do napisania programy do mnożenia macierzy na CPU i GPU na piątek (30.11.2012).",
				"30.11.2012", 4);
		Task t6 = new Task(
				"Spotkanie z Ukraińcami z Łucka",
				"W poniedziałek spotkanie w ANIRO w Sopocie z Ukraińcami z Łucka.",
				"26.11.2012", 5);

		storageService.addTask(t1);
		storageService.addTask(t2);
		storageService.addTask(t3);
		storageService.addTask(t4);
		storageService.addTask(t5);
		storageService.addTask(t6);

	}

}
